package app.controllers;

import javafx.fxml.FXML;
import javafx.scene.control.CheckBoxTreeItem;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;

import app.events.FileSelectedEvent;
import app.events.OpenProjectEvent;
import app.model.Model;
import javafx.fxml.Initializable;

public class FiletreeController implements Initializable, Controller {

  private EventBus eventBus;

  // Creating the images for the icons.
  Image folder = new Image(getClass().getResourceAsStream("/graphics/folder.png"));
  Image md = new Image(getClass().getResourceAsStream("/graphics/md.png"));
  Image java = new Image(getClass().getResourceAsStream("/graphics/java.png"));
  Image placeholder = new Image(getClass().getResourceAsStream("/graphics/placeholder.png"));

  // Creating the variable.
  @FXML
  private TreeView<String> filetree;

  @Override
  public void initialize(URL url, ResourceBundle resourceBundle) {
    //
  }

  @Override
  public void setEventBus(EventBus eventBus) {
    this.eventBus = eventBus;
    this.eventBus.register(this);
  }

  /**
   * The displaying of the fileTree. The inputChosen(the path) is aquired from the
   * eventBus (OpeFileProjectEvent). The root is created as a CheckBoxItems and
   * sends it to generateTree, and after that setting it to the root.
   */
  private void showTree(String inputChosen) {
    CheckBoxTreeItem<String> root = new CheckBoxTreeItem<>(inputChosen);
    filetree.setShowRoot(false);
    File fileInputChosen = new File(inputChosen);

    generateTree(fileInputChosen, root);

    filetree.setRoot(root);
  }

  /**
   * The method to generate the fileTree recursively. If it is a directory a
   * CheckBoxStringItem is created and the method is called again. It goes through
   * all until every directory or file inside the orginal CheckBoxItem is made. If
   * the item is a file it sends it to the help function checkExtension which is
   * described below.
   */
  private void generateTree(File file, CheckBoxTreeItem<String> parent) {

    if (file.isDirectory()) {
      CheckBoxTreeItem<String> element = new CheckBoxTreeItem<>(file.getName(), new ImageView(folder));
      parent.getChildren().add(element);

      List<File> dirList = new ArrayList<>();
      List<File> fileList = new ArrayList<>();

      sortFiles(dirList, fileList, file);

      for (File f : dirList) {
        generateTree(f, element);
      }

    } else {
      checkExtensions(file, parent);
    }
  }

  /**
   * A helping function to sort the files/folders in the fileTree so that it shows
   * in the correct order.
   */
  private void sortFiles(List<File> dirList, List<File> fileList, File file) {
    for (File f : file.listFiles()) {
      if (f.isDirectory())
        dirList.add(f);
      else {
        fileList.add(f);
      }

    }
    dirList.addAll(fileList);

  }

  /**
   * A help function to check if the extensions match .java or.md to insert the
   * spesific icons and sending it to another help funtion createExtension to
   * create the new CheckboxTreeItem<String> that will add to the parent.
   */
  private void checkExtensions(File file, CheckBoxTreeItem<String> parent) {
    String name = file.getName();
    String ext = (name.substring(file.getName().lastIndexOf(".") + 1, file.getName().length()));

    if ("java".equals(ext))
      createExtension(name, java, parent);
    else if ("md".equals(ext))
      createExtension(name, md, parent);
    else
      createExtension(name, placeholder, parent);
  }

  private void createExtension(String name, Image image, CheckBoxTreeItem<String> parent) {
    CheckBoxTreeItem<String> element = new CheckBoxTreeItem<>(name, new ImageView(image));
    parent.getChildren().add(element);
  }

  @FXML
  private void handleMouseClick(MouseEvent event) {
    if (event.getClickCount() == 2) {
      TreeItem<String> item = filetree.getSelectionModel().getSelectedItem();

      String root = Model.getProjectPath().getFileName().toString();
      String path = "";
      while (!root.equals(item.getValue())) {
        path = File.separator + item.getValue() + path;
        item = item.getParent();
      }

      path = Model.getProjectPath() + path;
      this.eventBus.post(new FileSelectedEvent(path));
    }
  }

  @Subscribe
  private void handle(OpenProjectEvent event) {
    this.showTree(event.getPath());
  }

}
