package app.controllers;

import java.io.File;
import java.net.URL;
import java.util.ResourceBundle;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;

import app.events.CopyEvent;
import app.events.CutEvent;
import app.events.ExitApplicationEvent;
import app.events.FileSelectedEvent;
import app.events.LanguageChangedEvent;
import app.events.OpenLinkInBrowserEvent;
import app.events.OpenProjectEvent;
import app.events.PasteEvent;
import app.events.RedoEvent;
import app.events.ThemeChangedEvent;
import app.events.ToggleCommentEvent;
import app.events.ToggleWrapTextEvent;
import app.events.UndoEvent;
import app.model.Model;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.CheckMenuItem;
import javafx.scene.control.MenuBar;
import javafx.scene.control.RadioMenuItem;
import javafx.scene.control.ToggleGroup;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import javafx.stage.Stage;

public class MenubarController implements Initializable, Controller {

  private EventBus eventBus;

  @FXML
  private MenuBar menubar;

  @FXML
  private ToggleGroup languageToggleGroup;

  @Override
  public void initialize(URL url, ResourceBundle resourceBundle) {
    // TODO: implement
  }

  @Override
  public void setEventBus(EventBus eventBus) {
    this.eventBus = eventBus;
    this.eventBus.register(this);
  }

  /* ------------------------------------------------------------------------ */
  /* FILE */
  /* ------------------------------------------------------------------------ */

  @FXML
  public void handleOpenFile() {
    FileChooser fc = new FileChooser();
    fc.setTitle("Open File");
    Stage stage = (Stage) menubar.getScene().getWindow();

    File chosenFile = fc.showOpenDialog(stage);
    String correctFormat = chosenFile.getAbsolutePath().replace("\\", "\\\\");

    Model.setActiveFilePath(chosenFile.toPath());
    this.eventBus.post(new FileSelectedEvent(correctFormat));
  }

  @FXML
  private void handleOpenProject() {
    DirectoryChooser dc = new DirectoryChooser();
    dc.setTitle("Open Project");
    Stage stage = (Stage) menubar.getScene().getWindow();

    File chosenDir = dc.showDialog(stage);
    String correctFormat = chosenDir.getAbsolutePath().replace("\\", "\\\\");

    Model.setProjectPath(chosenDir.toPath());
    this.eventBus.post(new OpenProjectEvent(correctFormat));

  }

  // @FXML
  // public void handleSaveFile() {
  // FileChooser fc = new FileChooser();
  // fc.setTitle("Save File");
  // Stage stage = (Stage) menubar.getScene().getWindow();

  // FileChooser.ExtensionFilter extentionjava = new
  // FileChooser.ExtensionFilter(".java");
  // FileChooser.ExtensionFilter extentionmd = new
  // FileChooser.ExtensionFilter(".md");
  // fc.getExtensionFilters().addAll(extentionjava, extentionmd);
  // fc.showOpenDialog(stage);
  // }

  /**
   * Handles the event where the language was change from the menu.
   * 
   * @param event
   */
  @FXML
  private void handleLanguageChange(ActionEvent event) {
    this.eventBus.post(new LanguageChangedEvent(((RadioMenuItem) event.getSource()).getText()));
  }

  @FXML
  private void handleToggleWraptext(ActionEvent event) {
    var isSelected = ((CheckMenuItem) event.getSource()).selectedProperty().get();
    this.eventBus.post(new ToggleWrapTextEvent(isSelected));
  }

  @FXML
  private void handleThemeChange(ActionEvent event) {
    this.eventBus.post(new ThemeChangedEvent(((RadioMenuItem) event.getSource()).getText()));
  }

  /**
   * Handles the event where there was an exit request from the menu.
   * 
   * @param event
   */
  @FXML
  private void handleExitApplication(ActionEvent event) {
    this.eventBus.post(new ExitApplicationEvent());
  }

  /* ------------------------------------------------------------------------ */
  /* EDIT */
  /* ------------------------------------------------------------------------ */

  @FXML
  private void handleUndo(ActionEvent event) {
    this.eventBus.post(new UndoEvent());
  }

  @FXML
  private void handleRedo(ActionEvent event) {
    this.eventBus.post(new RedoEvent());
  }

  @FXML
  private void handleCopy(ActionEvent event) {
    this.eventBus.post(new CopyEvent());
  }

  @FXML
  private void handleCut(ActionEvent event) {
    this.eventBus.post(new CutEvent());
  }

  @FXML
  private void handlePaste(ActionEvent event) {
    this.eventBus.post(new PasteEvent());
  }

  @FXML
  private void handleToggleComment(ActionEvent event) {
    this.eventBus.post(new ToggleCommentEvent());
  }

  /* ------------------------------------------------------------------------ */
  /* ABOUT */
  /* ------------------------------------------------------------------------ */

  @FXML
  private void handleAbout(ActionEvent event) {
    String aboutLink = "https://gitlab.stud.idi.ntnu.no/oysteikt/tdt4100-project-2021v/-/blob/master/README.md";
    this.eventBus.post(new OpenLinkInBrowserEvent(aboutLink));
  }

  /* ------------------------------------------------------------------------ */
  /* SUBSCRIPTIONS */
  /* ------------------------------------------------------------------------ */

  /**
   * Updates menubuttons whenever the language is changed
   * 
   * @param event
   */
  @Subscribe
  private void handle(LanguageChangedEvent event) {
    this.languageToggleGroup.getToggles().stream().map(RadioMenuItem.class::cast)
        .filter(t -> t.getId().equals("toggle" + event.getLanguage())).findFirst().orElseThrow().setSelected(true);
  }

}
